/*

	Import data from a Comma Separated Value (CSV) file
	
*/

#include "Hackdiet.h"

#ifdef CSV
#ifdef CSVDEBUG

#define MAXFIELDS	32

static int recno = 0;
static char *fields[MAXFIELDS];
static DmOpenRef dbp = NULL;
static MemHandle recHandle;
static char *dbdata = NULL;

static int ypos = 0;
static short lineHeight;
static RectangleType r;
static char csvRecord[132];
static Boolean eof = false, importOK;
static Boolean onerec;					// True if single record composite CSV database
static UInt16 nrecs, rindex = 0;			// Record count and index

extern int lastCSVfield;

//	drawLine  --  Draw text line in record gadget

static void drawLine(char *s)
{
	WinDrawChars(s, StrLen(s), r.topLeft.x, r.topLeft.y + ypos);
	ypos += lineHeight;

}

//	drawRecord  --  Draw the record gadget

#define TEST2
//#define TEST3

#ifdef TEST3
static Boolean newSearch = true;
static Err dbeof;
#endif

static void drawRecord(FormPtr fp, UInt16 gidget)
{
	UInt16 gudget = FrmGetObjectIndex(fp, gidget);
	
	#ifdef TEST1
	int i;
	char s[80], f[80];
#endif
	
#ifdef TEST2
	int year, month, day;
	weight wgt;
	rung rng;
	Boolean flag;
	char s[80];
	char ew[12];
	char cmt[132];
#endif
	
	lineHeight = FntLineHeight();

	FrmGetObjectBounds(fp, gudget, &r);
	WinEraseRectangle(&r, 0);
	ypos = 0;

#ifdef TEST2	
	if (dbp == NULL) {
		drawLine("No database.");
		return;
	}

	StrPrintF(s, "Record %d (nrecs = %d / %d)", recno, nrecs, rindex);
	drawLine(s);
	
	if (eof) {
		drawLine("End of file.");
		return;
	}
#endif
	drawLine(csvRecord);

#ifdef TEST2	
	if (parseDay(csvRecord, &year, &month, &day, &wgt, &rng, &flag, cmt)) {
		StrPrintF(s, "Date: %04d-%02d-%02d", year, month, day);
		drawLine(s);
		editWeight(ew, wgt);
		StrPrintF(s, "Weight: %s", ew);
		drawLine(s);
		StrPrintF(s, "Rung: %d", rng);
		drawLine(s);
		StrPrintF(s, "Flag: %d", flag);
		drawLine(s);
		StrPrintF(s, "Comment: %s", cmt);
		drawLine(s);
	} else {
		drawLine("Not a daily log record.");
		
		//	Oh, Hell, let's dump what he returned anyway
		
		StrPrintF(s, "Last field scanned: %d.", lastCSVfield);
		drawLine(s);
		StrPrintF(s, "Date: %04d-%02d-%02d", year, month, day);
		drawLine(s);
		editWeight(ew, wgt);
		StrPrintF(s, "Weight: %s", ew);
		drawLine(s);
		StrPrintF(s, "Rung: %d", rng);
		drawLine(s);
		StrPrintF(s, "Flag: %d", flag);
		drawLine(s);
	}
	drawLine(importOK ? "Import OK." : "Import failed.");
#endif
	
#ifdef TEST1
	CSVscanInit(csvRecord);
	
	i = 0;
	
	while (CSVscanField(f)) {
		i++;
		StrPrintF(s, "%2d \"%s\"", i, f);
		drawLine(s);
	}
#endif
}

/*	csvDebugHandleEvent  --  Handle event while the CSV debug form
							 is displayed.  */
							  

static Boolean csvDebugHandleEvent(EventPtr event)
{
	Boolean handled;
#ifdef __GNUC__
	CALLBACK_PROLOGUE
#endif
	handled = false;


	switch (event->eType) {
	  	
  		case frmOpenEvent:	
			FrmDrawForm(FrmGetActiveForm());
			drawRecord(FrmGetActiveForm(), CSVDebugRecordViewerGadget);
			importCSVstart();
			handled = true;
			break;
			
		case frmCloseEvent:
			FrmEraseForm(FrmGetActiveForm());
			if (dbp != NULL) {
				if (recHandle != NULL) {
					MemHandleUnlock(recHandle);
					recHandle = NULL;
				}
				DmCloseDatabase(dbp);
				dbp = NULL;
			}
			FrmEraseForm(FrmGetActiveForm());
			FrmDeleteForm(FrmGetActiveForm());
			handled = true;
			break;
			
		case ctlSelectEvent:
			switch(event->data.ctlSelect.controlID) {
				case CSVDebugDoneButton:			// Done button: Back to month list
#ifdef TEST2
					importCSVend();
#endif
					FrmGotoForm(MonthsForm);
					handled = true;
					break;

				case CSVDebugNextButton:			// Next button: Show next record
#ifdef TEST2
					if (dbp != NULL) {
						if (onerec) {
							eof = !csvRecordNext(csvRecord, sizeof csvRecord);
						} else {
							if (rindex < nrecs) {
								recHandle = DmQueryRecord(dbp, rindex);
								if (recHandle != NULL) {
									dbdata = MemHandleLock(recHandle);
									StrCopy(csvRecord, dbdata);
									MemHandleUnlock(recHandle);
									recHandle = NULL;
									eof = false;
									rindex++;								
								} else {
									eof = true;
								}
							} else {
								eof = true;
							}
						}
						if (!eof) {
							recno++;
							importOK = importCSVline(csvRecord);
						}
					}
#endif
#ifdef TEST3
					{
						static DmSearchStateType stateInfo;
						UInt16 cardno;
						LocalID dbid;
						
						dbeof = DmGetNextDatabaseByTypeCreator(newSearch,
									&stateInfo, ImportDbType, CreatorID, false,
									&cardno, &dbid);
						newSearch = false;
						if (dbeof == 0) {
							DmDatabaseInfo(cardno, dbid, csvRecord, NULL, NULL,
								NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
							csvRecord[33] = 0;
						} else {
							StrCopy(csvRecord, "No more databases");
							newSearch = true;
						}
					}
#endif
					drawRecord(FrmGetActiveForm(), CSVDebugRecordViewerGadget);
					handled = true;
					break;
			}
			break;
	}
#ifdef __GNUC__
	CALLBACK_EPILOGUE
#endif
	return handled;
}

/*	csvDebugLoad  --  Initialise CSV debug form. */

void csvDebugLoad(FormPtr f)
{
#ifdef TEST3
	newSearch = true;
#else
	dbp = DmOpenDatabaseByTypeCreator(ImportDbType, CreatorID, dmModeReadOnly);
	recHandle = NULL;
	if (dbp != NULL) {
		rindex = 0;
		nrecs = DmNumRecordsInCategory(dbp, dmAllCategories);
		
		/*	If the database contains a single record, we take it to be
			a complete CSV file embedded as a single binary chunk with
			a 4 byte length at the start of the chunk.  Otherwise, we
			assume this to be a text mode embedded database with one
			record per line of CSV file.  This complication is
			unnecessary, as either mode would be sufficient, but it
			allows testing both embedding modes of the desktop
			"pdbmake" utility.  */
		
		if ((onerec = (nrecs == 1)) == true) {
			recHandle = DmQueryNextInCategory(dbp, &rindex, dmAllCategories);
			if (recHandle != NULL) {
				dbdata = MemHandleLock(recHandle);
				csvRecordInit(dbdata + sizeof(long), *((long *) dbdata));
			}
		}
	}
	csvRecord[0] = 0;
	recno = 0;
#endif
	eof = false;
	FrmSetEventHandler(f, csvDebugHandleEvent);
}

#endif
#endif