/*
	
					The Hacker's Diet

					 Database Manager				
*/

#include "Hackdiet.h"

#define CompressedComment	32				// Compressed comment flag in day number

int hdFoundIndex = -1;						// Database	index of last record found by various functions	

//	updateTrend  --  Calculate the day per day trend in an unpacked month

void updateTrend(struct hdMonth *m)
{
	UInt16 i;
	trend t = m->trendCarryForward;
	signed long nt;

	/*	If trend carryforward is zero, try to fill it in from the
		data of the last month.  If that fails, use the first supplied
		date in this month, but don't set the permanent carryforward,
		as we may later obtain a more accurate value.  */
		
	if (t == 0) {
		int iter = -1, bnrec = -1;
		UInt16 month, year, nrec = 0;
		struct hdMonth *lm;
		
		while (true) {
			hdIterateMonths(&iter, &year, &month);
			if (iter == -1) {
				break;
			}
			if ((year > m->year) || ((year == m->year) && (month >= m->month))) {
				iter = -1;
				hdIterateMonths(&iter, &year, &month);
				break;
			}
			bnrec = nrec;
			nrec++;
		}
		if (bnrec >= 0) {
			lm = hdGetMonthUnpacked(bnrec);		// Note that this may call updateTrend recursively
			t = m->trendCarryForward = cvtTrend(lm->trendArray[lm->days - 1], (weightunit) lm->weightUnit, (weightunit) m->weightUnit);
			hdDisposeMonthUnpacked(lm);
		} else {
			for (i = 0; i < m->days; i++) {
				if (m->weightArray[i] != 0) {
					t = WeightToTrend(m->weightArray[i]);
					break;
				}
			}
		}
	}
#ifdef OLDWAY	
	m->trendArray[0] = t;
	for (i = 1; i < m->days; i++) {
		if (m->weightArray[i] != 0) {
			/* Note that the fact we store weights in hundredths
			   of a unit means we can compute the exponentially
			   smoothed moving average entirely in integers. */
			nt = ((signed long) t) + (((signed long) WeightToTrend(m->weightArray[i])) - ((signed long) t)) / 10;
			t = (trend) nt;
		}
		m->trendArray[i] = t;
	}
#else
	for (i = 0; i < m->days; i++) {
		if (m->weightArray[i] != 0) {
			/* Note that the fact we store weights in hundredths
			   of a unit means we can compute the exponentially
			   smoothed moving average entirely in integers. */
			nt = ((signed long) t) + (((signed long) WeightToTrend(m->weightArray[i])) - ((signed long) t)) / 10;
			t = (trend) nt;
		}
		m->trendArray[i] = t;
	}
#endif
}

/*	unpackMonthRecord  --  Unpack a database record for a month and
						   return a dynamically allocated hdMonth
						   structure containing the data.  */
						   
struct hdMonth *unpackMonthRecord(struct hdMonthlyLog *md)
{
	MemHandle mh;
	struct hdMonth *m = NULL;
	UInt16 i, j, k, commentLength;
	unsigned char c;
	char *cp;
	
	//	Determine length of comment text
	
	commentLength = 0;
	i = 0;
	while (md->commentArray[i] != 0) {
		if ((md->commentArray[i] & CompressedComment) != 0) {
			/*	Since compressed comments re-use pointers to comments
				on earlier days, there's no need to reserve space for
				them in the commentData of the unpacked record.  */
			i += 2;
		} else {
			commentLength += md->commentArray[i + 1] + 1;
			i += md->commentArray[i + 1] + 2;
		}
	}
	
	/*	We have to use a movable handle so to permit hdAddComment()
		to expand the buffer when necessary.  */
	
	mh = MemHandleNew(sizeof(struct hdMonth) + commentLength);
	if (mh != 0) {
		m = MemHandleLock(mh);
	}
	
	if (m != NULL) {
	
		m->dataLength = sizeof(struct hdMonth) + commentLength;
	
		//	Transfer scalar information
		
		m->year = md->year;
		m->month = md->month;
		m->trendCarryForward = md->trendCarryForward;
		m->weightUnit = md->weightUnit;
		m->lastModificationTime = md->lastModificationTime;
		
		m->days = DaysInMonth(m->month + 1, m->year);		// Days in this month
		m->firstDay = DayOfWeek(m->month + 1, 1, m->year);	// Weekday of first day of month
		
		//	Copy weight and rung arrays
		
		for (i = 0; i < m->days; i++) {
			m->weightArray[i] = md->weightArray[i];
			m->rungArray[i] = md->rungArray[i];
		}
		updateTrend(m);					// Calculate trendArray
		
		//	Unpack flag bit vector into easy-to-index array
		
		for (i = j = k = 0; i <= m->days; i++) {
			if (j == 0) {
				c = md->flagsArray[k++];
			}
			j = (j + 1) & 7;
			m->flagsArray[i] = (c & 0x80) ? 1 : 0;
			c <<= 1;
		}
		
		//	Clear out comment pointer array
		
		MemSet(m->commentArray, (m->days) * (sizeof (m->commentArray[0])), 0);
		
		/*	Walk through comment text, transfer comments to our
			commentData table and set pointers to the proper days
			in commentArray.  Compressed comments simply copy the
			offset of the earlier duplicate comment.  (Note that
			compressed comments must always refer to a prior day's
			comment, not a subsequent one.  */
			
		i = 0;
		cp = m->commentData;
		while (md->commentArray[i] != 0) {
			if ((md->commentArray[i] & CompressedComment) != 0) {
				m->commentArray[(md->commentArray[i] & (CompressedComment - 1)) - 1] =
					m->commentArray[md->commentArray[i + 1] - 1];
				i += 2;
			} else {
				m->commentArray[md->commentArray[i] - 1] = cp - ((char *) m);
				MemMove(cp, (md->commentArray) + i + 2, md->commentArray[i + 1]);
				cp += md->commentArray[i + 1];
				*cp++ = 0;
				i += md->commentArray[i + 1] + 2;
			}
		}			
		
	}
	
	return m;
}						 

/*	packMonthRecord  --  Pack data for a month into a compact database
						 record for storage.  */
						 
static struct hdMonthlyLog *packMonthRecord(struct hdMonth *m, UInt16 *ml)
{
	struct hdMonthlyLog *md;
	UInt16 d, i, j, k, commentLength;
	unsigned char f;
	char *cmt;
	/*	The compressed array for a given day is 0 if there is no
		comment for that day or the text of the comment does not
		appear in an earlier day or the day number (1 - 31) of a
		prior day whose comment is identical to this one.  */
	char compressed[32];
	
	/*	Walk through the comments and determine their packed size.
		In the process, we check for comments identical to those
		which figure earlier in the month and mark them in the
		compressed[] array to re-use the earlier text.  */
	
	MemSet(compressed, sizeof compressed, 0);
	commentLength = 1;						// Start with zero terminator byte
	for (i = 0; i < m->days; i++) {
		if (m->commentArray[i] != NULL) {
			int j;
			
			for (j = 0; j < i; j++) {
				if ((m->commentArray[j] != NULL) &&
					(StrCompare(((char *) m) + m->commentArray[i], ((char *) m) + m->commentArray[j]) == 0)) {
					compressed[i] = (char) (j + 1);
					break;
				}
			}
			
			if (compressed[i] != 0) {
				commentLength += 2;
			} else {
				commentLength += StrLen(((char *) m) + m->commentArray[i]) + 2;
			}
		}
	}
	
	//	Allocate monthly log record
	
	md = MemPtrNew(*ml = ((sizeof(struct hdMonthlyLog) - 2) + commentLength));
	
	if (md != NULL) {
		
		//	Transfer scalar values
		
		md->year = m->year;
		md->month = m->month;
		md->trendCarryForward = m->trendCarryForward;
		md->weightUnit = m->weightUnit;
		md->lastModificationTime = m->lastModificationTime;
		
		md->padByte1 = 0;					// Guarantee pad byte after run table always zero
		
		//	Copy weight and rung arrays
		
		for (i = 0; i < m->days; i++) {
			md->weightArray[i] = m->weightArray[i];
			md->rungArray[i] = m->rungArray[i];
		}
		
		//	Pack flag array into a bit vector
		
		k = 0;
		for (i = 0; i < 4; i++) {
			f = 0;
			for (j = 0; j < 8; j++) {
				d = (k < m->days) ? m->flagsArray[k] : 0;
				f = (f << 1) | (d != 0);
				k++;	
			}
			md->flagsArray[i] = f;
		}
		
		/*	Pack comments into comment vector.  Comments which duplicate
			those earlier in the month will have already been marked as
			such in the compressed[] array and are stored as two-byte
			references to the prior comments.  */
		
		cmt = md->commentArray;
		for (i = 0; i < m->days; i++) {
			if (m->commentArray[i] != 0) {
				if (compressed[i] != 0) {
					*cmt++ = (char) (CompressedComment | (i + 1));
					*cmt++ = compressed[i];
				} else {
					UInt16 l = StrLen(((char *) m) + m->commentArray[i]);
					
					*cmt++ = (char) (i + 1);
					*cmt++ = (char) l;
					MemMove(cmt, ((char *) m) + m->commentArray[i], l);
					cmt += l;
				}
			}
		}
		*cmt++ = 0;						// Append zero terminator to comment array
	}
	
	return md;
}

/*	hdAddComment  --  Add a new comment to an unpacked month array,
					  expanding it in memory as required.  Note
					  that due to possible movement, the hdMonth
					  pointer you get on return may not be the
					  same as the one you passed.  If the block
					  cannot be expanded, a pointer to the unmodified
					  hdMonth item (which may have moved if the heap
					  was compacted by the attempt to resize) is
					  returned.  */
					  
struct hdMonth *hdAddComment(struct hdMonth *m, int day, char *comment)
{
	MemHandle mh;
	int nl;
	char *newm;
	
	/*	This is complicated somewhat by the need to unlock the month
		data before expanding it to include the new comment text
		so that it may be moved, if necessary, in order to be reallocated
		with the new size.  We don't attempt to recover memory from a
		previous comment, assuming that no reasonable user is going
		to make so many changes to comments before leaving the month
		so as to exhaust memory.  If for some reason this becomes a
		problem one could always pack the month and then unpack it
		if the MemHandleResize fails, which will purge obsolete
		comments.  */

	assert(StrLen(comment) <= MAX_COMMENT);	
	nl = m->dataLength + StrLen(comment) + 1;	// Length to include new string
	mh = MemPtrRecoverHandle(m);
	if (mh != 0) {
		MemHandleUnlock(mh);
		if (MemHandleResize(mh, nl) != 0) {
			m = MemHandleLock(mh);				// Out of memory--discard change
			return m;
		}
		newm = MemHandleLock(mh);
		m = (struct hdMonth *) newm;
		StrCopy(newm + m->dataLength, comment);	// Copy comment to end of table
		m->commentArray[day] = m->dataLength;	// Set relative comment pointer for day
		m->dataLength = nl;						// Add new comment to data length
	} else {
		return m;
	}
	return m;
}

//	newMonth  --  Create new blank unpacked month for a given year and month

static struct hdMonth *newMonth(int year, int month)
{
	MemHandle mh;
	struct hdMonth *m;
	UInt16 i;
	
	assert(month >= 0 && month < 12);
	mh = MemHandleNew(sizeof(struct hdMonth));
	if (mh == 0) {
		return NULL;
	}
	m = MemHandleLock(mh);

	m->year = year;
	m->month = month;
	
	m->trendCarryForward = 0;		// Clear trend carry forward (caller must fill in)
	m->weightUnit = prefs.weightUnit;
	m->lastModificationTime = TimGetSeconds();
		
	m->days = DaysInMonth(m->month + 1, m->year);		// Days in this month
	m->firstDay = DayOfWeek(m->month + 1, 1, m->year);	// Weekday of first day of month
		
	//	Initialise arrays for days
		
	for (i = 0; i < m->days; i++) {
		m->weightArray[i] = 0;
		m->rungArray[i] = 255;
		m->flagsArray[i] = 0;
		m->commentArray[i] = 0;
	}
	m->commentData[0] = 0;
	
	return m;
}

//	hdNumMonths  --  Return number of months present in database

UInt16 hdNumMonths(void)
{
	DmOpenRef dbp;
	UInt16 titems = 0;
	
	dbp = DmOpenDatabaseByTypeCreator(MonthDbType, CreatorID, dmModeReadOnly);
	if (dbp != NULL) {
		titems = DmNumRecordsInCategory(dbp, dmAllCategories);
		DmCloseDatabase(dbp);
	}
	return titems;
}

/*	hdIterateMonths  --  Walk through months in the database and return
						 record number, year, and month for each.  "iter"
						 is a state variable which must be set to -1 before
						 the first call and -1 if the caller wishes to
						 terminate the iteration.  A year number of 0 is
						 returned when the end of the database is encountered.  */
						 
void hdIterateMonths(int *iter, UInt16 *year, UInt16 *month)
{
	static DmOpenRef dbp = NULL;
	static UInt16 recnr, titems;
	MemHandle rh;
	
	*year = 0;
	if (*iter == -1) {
		if (dbp != NULL) {
			DmCloseDatabase(dbp);
			dbp = NULL;
			return;
		}
		dbp = DmOpenDatabaseByTypeCreator(MonthDbType, CreatorID, dmModeReadOnly);
		if (dbp == NULL) {
			return;				// iter remains -1 as error indicator
		}
		*iter = 0;
		recnr = 0;
		titems = DmNumRecordsInCategory(dbp, dmAllCategories);
	}
	if (recnr < titems) {
		rh = DmQueryNextInCategory(dbp, &recnr, dmAllCategories);
		if (rh != NULL) {
			struct hdMonthlyLog *md = MemHandleLock(rh);
			
			*year = md->year;
			*month = md->month;
			hdFoundIndex = recnr;
			MemHandleUnlock(rh);
			recnr++;
			*iter++;
		} else {
			DmCloseDatabase(dbp);
			dbp = NULL;
			hdFoundIndex = -1;
			*iter = -1;
		}
	} else {
		DmCloseDatabase(dbp);
		dbp = NULL;
		*iter = -1;
	}
}

//	hdFindMonth  --  Find entry for a given year and month and return database record number

int hdFindMonth(UInt16 dyear, UInt16 dmonth)
{
	int iter = -1;
	UInt16 month, year;
	UInt16 recnr = 0;
	
	while (true) {
		hdIterateMonths(&iter, &year, &month);
		if (iter == -1) {
			break;
		}
		if ((dyear == year) && (dmonth == month)) {
			iter = -1;
			hdIterateMonths(&iter, &year, &month);
			return recnr;
		}
		recnr++;
	}
	assert(false);
	return -1;					//	Can't happen
}

//	hdGetMonthUnpacked  --  Retrieve a month's record from the database and return unpacked

struct hdMonth *hdGetMonthUnpacked(UInt16 recnr)
{
	DmOpenRef dbp = NULL;
	MemHandle rh;
	UInt16 rindex = recnr;
	struct hdMonth *m = NULL;
	
	dbp = DmOpenDatabaseByTypeCreator(MonthDbType, CreatorID, dmModeReadOnly);
	if (dbp != NULL) {
		rh = DmQueryNextInCategory(dbp, &rindex, dmAllCategories);
		if (rh != NULL) {
			struct hdMonthlyLog *md = MemHandleLock(rh);
			
			m = unpackMonthRecord(md);
			MemHandleUnlock(rh);
			
		}
		DmCloseDatabase(dbp);
	}
	return m;
}

/*	hdGetMonthByDate  --  Find and retrieve a month's record given year and date,
						  optionally (create == true) creating if not already present.
						  If the create flag is false and the month is not present
						  in the database, NULL is returned.  */
						  
struct hdMonth *hdGetMonthByDate(UInt16 year, UInt16 month, UInt16 *recnr, Boolean create)
{
	int rec;
	
	if (create) {
		hdEnsureMonthPresent(year, month);
	}
	if (!hdIsMonthPresent(year, month)) {
		return false;
	}
	rec = hdFindMonth(year, month);
	*recnr = rec;
	return hdGetMonthUnpacked(rec);
}

//	hdPutMonthUnpacked  --  Modify a month's record to reflect changes to an unpacked month

Boolean hdPutMonthUnpacked(struct hdMonth *m, UInt16 recnr)
{
	UInt16 mdl;
	struct hdMonthlyLog *md;
	DmOpenRef dbp = NULL;
	MemHandle rh;
	Boolean ok = false;
	
	dbp = DmOpenDatabaseByTypeCreator(MonthDbType, CreatorID, dmModeReadWrite);
	if (dbp != NULL) {	
		md = packMonthRecord(m, &mdl);
		if (md != NULL) {
			rh = DmResizeRecord(dbp, recnr, mdl);
			if (rh != NULL) {
				MemPtr rp = MemHandleLock(rh);
				
				if (rp != NULL) {
					md->lastModificationTime = TimGetSeconds();
					DmWrite(rp, 0, md, mdl);
					MemHandleUnlock(rh);
					ok = true;
				}
			}
			MemPtrFree(md);
		}
		DmCloseDatabase(dbp);
	}
	return ok;
}

//	hdDisposeMonthUnpacked  --  Release memory for an unpacked month

void hdDisposeMonthUnpacked(struct hdMonth *m)
{
	MemHandle mh = MemPtrRecoverHandle(m);

	MemHandleUnlock(mh);
	MemHandleFree(mh);
}

/*	hdSaveMonthUnpacked  --  Store an unpacked month in the database, creating
							 a new record in the correct chronological order if
							 necessary.  */
							 
Boolean hdSaveMonthUnpacked(struct hdMonth *m)
{
	int iter = -1;
	UInt16 recnr = 0, year, month;
	Boolean ok = false;
	
	while (true) {
		hdIterateMonths(&iter, &year, &month);
		if ((iter == -1) || (m->year < year) || ((m->year == year) && (m->month < month))) {
			UInt16 ml;
			struct hdMonthlyLog *md;
			DmOpenRef dbp = NULL;
		
			/*	This record is later than the one we're saving.
				Insert a new record before this one.  */
			if (iter != -1) {						// If we didn't hit the end, we need to terminate iteration
				UInt16 yy, mm;
				iter = -1;	
				hdIterateMonths(&iter, &yy, &mm);	// Terminate iteration over database
			}
			dbp = DmOpenDatabaseByTypeCreator(MonthDbType, CreatorID, dmModeReadWrite);
			if (dbp != NULL) {
				md = packMonthRecord(m, &ml);
					
				if (md != NULL) {
					MemHandle rh;
					MemPtr rd;
					
					md->lastModificationTime = TimGetSeconds();
					rh = DmNewRecord(dbp, &recnr, ml);
					rd = MemHandleLock(rh);
					DmWrite(rd, 0, md, ml);
					MemHandleUnlock(rh);
					MemPtrFree(md);
					DmReleaseRecord(dbp, recnr, true);
				}
				DmCloseDatabase(dbp);
				ok = true;
			}
			break;
		}
		
		if ((year == m->year) && (month == m->month)) {
			//	Record is already in the database--update it
			ok = hdPutMonthUnpacked(m, recnr);
			break;
		}
		
		recnr++;
	}
	return ok;
}

//	hdIsMonthPresent  --  Check whether a month is present in the database

Boolean hdIsMonthPresent(UInt16 dyear, UInt16 dmonth)
{
	int iter = -1;
	Boolean present = false;
	UInt16 month, year;
	
	while (true) {
		hdIterateMonths(&iter, &year, &month);
		if ((year == 0) || (iter == -1)) {
			break;
		}
		if ((dyear == year) && (dmonth == month)) {
			iter = -1;
			hdIterateMonths(&iter, &year, &month);
			present = true;
			break;
		}
	}
	return present;
}

/*	hdMonthsPresent  --  Set a 12 element Boolean array to indicate which months
						 for a given year are present in the database.  You could
						 do this with hdIsMonthPresent(), but this function is
						 much faster because it scans months after the one
						 found by database index and doesn't open and close the
						 database for each query.  As an extra gimmick, since
						 folks likely to call this function would probably be
						 intensely interested and we're going to discover anyway,
						 the previousYear and nextYear in the database are returned
						 through the eponymous pointers, or -1 if the requested
						 year is at at end of the database.  You can pass NULL to
						 the adjacent year pointers if you aren't interested in
						 receiving that information.  */
						 
void hdMonthsPresent(int year, Boolean monthsPresent[12], int *previousYear, int *nextYear)
{
	DmOpenRef dbp = NULL;
	UInt16 recnr, titems;
	MemHandle rh;
	int pyear = -1, nyear = -1;
	
	MemSet(monthsPresent, sizeof(Boolean) * 12, 0);
	if (previousYear != NULL) {
		*previousYear = -1;
	}
	if (nextYear != NULL) {
		*nextYear = -1;
	}
	
	dbp = DmOpenDatabaseByTypeCreator(MonthDbType, CreatorID, dmModeReadOnly);
	if (dbp == NULL) {
		return;
	}
	recnr = 0;
	titems = DmNumRecordsInCategory(dbp, dmAllCategories);
	
	while (recnr < titems) {
		rh = DmQueryNextInCategory(dbp, &recnr, dmAllCategories);
		if (rh != NULL) {
			struct hdMonthlyLog *md = MemHandleLock(rh);
			int ryear = md->year, rmonth = md->month;
			
			MemHandleUnlock(rh);
			recnr++;
			
			if (ryear == year) {
				monthsPresent[rmonth] = true;
				while (recnr < titems) {
					rh = DmQueryNextInCategory(dbp, &recnr, dmAllCategories);
					if (rh != NULL) {
						md = MemHandleLock(rh);
						ryear = md->year;
						rmonth = md->month;
						MemHandleUnlock(rh);
						if (ryear == year) {
							monthsPresent[rmonth] = true;
						} else {
							nyear = ryear;
							recnr = titems;					// Bail out at start of next year
						}
						recnr++;
					}
				}
			} else if (ryear < year) {
				pyear = ryear;
			}
		}
	}
	DmCloseDatabase(dbp);
	dbp = NULL;
	if (previousYear != NULL) {
		*previousYear = pyear;
	}
	if (nextYear != NULL) {
		*nextYear = nyear;
	}
}
						 

/*	hdEnsureMonthPresent  --  Check whether a month is in the database and add
							  a blank record for it if it isn't.  */
							  
void hdEnsureMonthPresent(UInt16 dyear, UInt16 dmonth)
{
	if (!hdIsMonthPresent(dyear, dmonth)) {
		struct hdMonth *m;
		
		m = newMonth(dyear, dmonth);
		//	Get trend carryforward from preceding month and fill in new month
		hdSaveMonthUnpacked(m);
		hdDisposeMonthUnpacked(m);
	}
}

/*	hdEnsureDatabasePresent  --  Create the database if it doesn't currently exist.
								 Returns true if a new database was created.  */

Boolean hdEnsureDatabasePresent(void)
{
	DmOpenRef dbp;
	Boolean created = false;
	
	dbp = DmOpenDatabaseByTypeCreator(MonthDbType, CreatorID, dmModeReadOnly);
	if (dbp == NULL) {
		DmCreateDatabase(0, MonthDbName, CreatorID, MonthDbType, false);

#ifdef BACKUPDB
		
		//	Set backup bit on database
		
		dbp = DmOpenDatabaseByTypeCreator(MonthDbType, CreatorID, dmModeReadOnly);
		if (dbp != NULL) {
			LocalID lid;
			UInt16 attr, card;
			
			DmOpenDatabaseInfo(dbp, &lid, NULL, NULL, &card, NULL);
			DmDatabaseInfo(card, lid, NULL, &attr, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
			attr |= dmHdrAttrBackup;
			DmSetDatabaseInfo(card, lid, NULL, &attr, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
			DmCloseDatabase(dbp);
		}
#endif
		created = true;
	} else {
		DmCloseDatabase(dbp);
	}
	return created;
}

/*  hdFindInComments  --  Determine whether the search target occurs in
						  any of the comments in this unpacked record.  If
						  so, returns a UInt32 into which the year, month, and
						  day is packed.  If no comment contains the target
						  string, zero is returned.  This function is called
						  by the Find handler, and works on packed database
						  records in the interest of speed.  Further note that
						  it is not permitted to reference any application global
						  variables because they don't exist when Find invokes
						  us while another application is active.  */
						  
UInt32 hdFindInComments(struct hdMonthlyLog *md, char *target, UInt16 * pos)
{
	int i;
	char s[256];
			
	i = 0;
	while (md->commentArray[i] != 0) {
		if ((md->commentArray[i] & CompressedComment) != 0) {
			/*	We just skip compressed comments since we already know
				the earlier instance this is identical to doesn't match.  */
			i += 2;
		} else {
			assert(md->commentArray[i + 1] <= MAX_COMMENT);
			MemMove(s, (md->commentArray) + i + 2, md->commentArray[i + 1]);
			s[md->commentArray[i + 1]] = 0;
			if (FindStrInStr(s, target, pos)) {
				//	Find: return encoded location (guaranteed to be non-zero)
				return (((UInt32) md->year) << 16) | (md->month << 8) | md->commentArray[i];
			}
			i += md->commentArray[i + 1] + 2;
		}
	}			
	return 0;
}

/*	hdWeightExtrema  --  Determine weight and trend extrema for database
						 records in the given date range.  This is used
						 by History.c to auto-scale its charts to fit the
						 screen and, since a chart can cover a potentially
						 large number of months, this is written to be as
						 fast as possible, operating on packed months in the
						 database.  Note that no trend value in a month can
						 exceed the extrema of the trend carry-forward and
						 weight values in the month (the proof of this is
						 left for the reader), and hence we don't have to
						 evaluate the trend for each day in the month, which
						 saves a lot of time.
						 
						 Since individual months in the database may use
						 different weight units, we need to perform this
						 computation in uniform units.  Since a function
						 like this is usually called when preparing information
						 for the user, particularly auto-scaling charts, we
						 convert everything to the current display units and
						 return the results in the display unit.  */

void hdWeightExtrema(UInt16 syear, UInt16 smonth, UInt16 eyear, UInt16 emonth,
					 weight *weightMin, weight *weightMax)
{
	weight wmin = WEIGHT_MAX,
		   wmax = 0,
		   mmin, mmax;
	static DmOpenRef dbp = NULL;
	static UInt16 recnr, titems;
	MemHandle rh;
	int i, ndays;
	
	dbp = DmOpenDatabaseByTypeCreator(MonthDbType, CreatorID, dmModeReadOnly);
	if (dbp != NULL) {
		recnr = 0;
		titems = DmNumRecordsInCategory(dbp, dmAllCategories);
		while (recnr < titems) {
			rh = DmQueryNextInCategory(dbp, &recnr, dmAllCategories);
			if (rh != NULL) {
				struct hdMonthlyLog *md = MemHandleLock(rh);
			
				//	Determine whether this month is within the requested range
				
				if (((md->year > syear) && (md->year < eyear)) ||		// Entire year in range
					((md->year == syear) && (md->month >= smonth) &&
					 ((md->year < eyear) || ((md->year == eyear) && (md->month <= emonth)))) ||	// Within starting year
					((md->year == eyear) && (md->month <= emonth) &&
					 ((md->year > syear) || ((md->year == syear) && (md->month >= smonth))))	// Within ending year
				   ) {
				  	if (md->trendCarryForward != 0) {
				  		wmin = min(wmin, TrendToWeight(cvtTrend(md->trendCarryForward, (weightunit) (md->weightUnit), prefs.displayUnit)));
				  		wmax = max(wmax, TrendToWeight(cvtTrend(md->trendCarryForward, (weightunit) (md->weightUnit), prefs.displayUnit)));
				  	}
				  	ndays = DaysInMonth(md->month + 1, md->year);
				  	mmin = WEIGHT_MAX;
				  	mmax = 0;
				  	for (i = 0; i < ndays; i++) {
				  		if (md->weightArray[i] != 0) {
				  			mmin = min(mmin, md->weightArray[i]);
				  			mmax = max(mmax, md->weightArray[i]);
				  		}
				  	}
				  	if (mmax > 0) {
				  		wmin = min(wmin, cvtWeight(mmin, (weightunit) (md->weightUnit), prefs.displayUnit));
				  		wmax = max(wmax, cvtWeight(mmax, (weightunit) (md->weightUnit), prefs.displayUnit));
				  	}
				} else {
					/*	Month isn't in range.  If it's past the end of the
						range, bail out immediately.  */
					if ((md->year > eyear) || ((md->year == eyear) && (md->month > emonth))) {
						recnr = titems;
					}
				}
				MemHandleUnlock(rh);
			}
			recnr++;
		}
		DmCloseDatabase(dbp);
	}
		   
	*weightMin = wmin;
	*weightMax = wmax;
}

/*	hdPropagateTrend  --  When a change is made in the database which affects
						  calculation of the trend for subsequent months, propagate
						  the trend-carryover to those months, recalculating the final
						  trend for each before copying to the next.  This may seem
						  a horribly compute-bound process to deploy on a handheld
						  but in reality it will happen only in the rarest of
						  circumstances: when import or entry of data in months
						  prior to the current month requires revision of subsequent
						  months in the database.  Note that we work in terms of database
						  record numbers rather than dates.  This is not only more
						  efficient, it's essential to correctly handle the case
						  where one or more months' data are missing from the database.  */
						  
void hdPropagateTrend(UInt16 year, UInt16 month)
{
	struct hdMonth *m;
	UInt16 recnr, titems, i;
	Boolean first = true;
	
	titems = hdNumMonths();
	recnr = hdFindMonth(year, month);
	
	for (i = recnr; i < titems; i++) {
		m = hdGetMonthUnpacked(i);
		assert(m != NULL);
		if (m != NULL) {
			m->trendCarryForward = 0;		// Wipe existing carryforward
			updateTrend(m);					// Update trend for this month
			hdPutMonthUnpacked(m, i);
			hdDisposeMonthUnpacked(m);
		}
	}
}			   
