/*
	
					The Hacker's Diet

				   Exercise Ladder Form
					 
		This file implements a digital reference document
		for the exercise ladder presented in The Hacker's
		Diet.  It is linked in various ways to the monthly
		log in the interest of convenience.
			
*/

#include "Hackdiet.h"

/*	The following array contains the data for both the
	introductory and lifetime exercise ladders.  The
	data are stored in one large array eliminate the need
	for constant pointers to sub-arrays which would cost
	memory.  The table can be accessed conveniently through
	the following macros.  */

#define C_Rung			0	
#define C_Bend			1
#define	C_Sit_up		2
#define C_Leg_lift		3
#define C_Push_up		4
#define C_Steps			5
#define C_Count			6

#define nColumns		7

#define Exercise(rung, column)	(exercise[((nColumns - 1) * ((rung) - 1)) + ((column) - 1)])

#define nRungs		   48				// Rungs in table
#define nLifetime	   16				// Rung where lifetime ladder starts
#define nTable		   (nRungs + 2)		// Items in table (rungs and labels)

//	Transformations between rung numbers and exercise table indices

#define	RungToIndex(r)	((r) + (((r) >= nLifetime) ? 1 : 0))
#define IndexToRung(i)	(((i) <= nLifetime) ? (i) : ((i) - 1))

//	Values in Bend field which denote section labels for table

#define isLabel(r)		(Exercise(r, C_Bend) > 250)	
#define IntroductoryLabel	254
#define LifetimeLabel		255

static unsigned const char exercise[] = {

//					   Introductory Ladder

/* Label */	IntroductoryLabel, 0, 0, 0, 0, 0,

// Rung 	 Bend  Sit up  Leg lift  Push up  Steps  Count
/*	  1 */	   2,	  3,	   4,		2,		1,	   30,
/*	  2 */	   3,	  4,	   5,		3,		1,	   65,
/*	  3 */	   4,	  6,	   6,		3,		2,	   20,
/*	  4 */	   6,	  7,	   8,		4,		2,	   50,
/*	  5 */	   7,	  9,	   9,		5,		3,		0,
/*	  6 */	   8,	 10,	  10,		6,		3,	   30,
/*	  7 */	  10,	 11,	  12,		7,		3,	   55,
/*	  8 */	  12,	 13,	  14,		8,		4,		5,
/*	  9 */	  14,	 15,	  16,		9,		4,	   25,
/*	 10 */	  16,	 16,	  18,	   11,		4,	   50,
/*	 11 */	  18,	 18,	  20,	   12,		4,	   70,
/*	 12 */	  20,	 20,	  22,	   13,		5,	   15,
/*	 13 */	  23,	 21,	  25,	   15,		5,	   30,
/*	 14 */	  25,	 23,	  27,	   16,		5,	   50,
/*	 15 */	  28,	 25,	  30,	   18,		5,	   65,

//						 Lifetime Ladder

/* Label */	LifetimeLabel, 0, 0, 0, 0, 0,

// Rung 	 Bend  Sit up  Leg lift  Push up  Steps  Count

/*	 16 */	  14,	 10,	  12,		9,		4,	   40,
/*	 17 */	  15,	 11,	  14,	   10,		4,	   55,
/*	 18 */	  16,	 12,	  16,	   11,		5,		0,
/*	 19 */	  18,	 13,	  17,	   12,		5,	   15,
/*	 20 */	  19,	 14,	  19,	   13,		5,	   30,
/*	 21 */	  21,	 15,	  21,	   14,		5,	   45,
/*	 22 */	  22,	 16,	  23,	   15,		5,	   60,
/*	 23 */	  24,	 17,	  25,	   16,		5,	   70,
/*	 24 */	  25,	 18,	  27,	   17,		6,	   10,
/*	 25 */	  27,	 20,	  29,	   18,		6,	   20,
/*	 26 */	  29,	 21,	  31,	   19,		6,	   30,
/*	 27 */	  31,	 23,	  33,	   20,		6,	   40,
/*	 28 */	  33,	 24,	  36,	   21,		6,	   50,
/*	 29 */	  34,	 26,	  38,	   22,		6,	   60,
/*	 30 */	  36,	 28,	  40,	   23,		6,	   65,
/*	 31 */	  38,	 29,	  43,	   24,		7,		0,
/*	 32 */	  40,	 31,	  45,	   25,		7,		5,
/*	 33 */	  43,	 33,	  48,	   26,		7,	   10,
/*	 34 */	  45,	 35,	  51,	   27,		7,	   15,
/*	 35 */	  47,	 37,	  54,	   28,		7,	   15,
/*	 36 */	  49,	 39,	  56,	   29,		7,	   20,
/*	 37 */	  51,	 41,	  59,	   30,		7,	   20,
/*	 38 */	  54,	 43,	  62,	   31,		7,	   20,
/*	 39 */	  56,	 46,	  65,	   32,		7,	   25,
/*	 40 */	  59,	 48,	  68,	   33,		7,	   30,
/*	 41 */	  61,	 50,	  72,	   34,		7,	   30,
/*	 42 */	  64,	 53,	  75,	   35,		7,	   30,
/*	 43 */	  66,	 55,	  78,	   36,		7,	   35,
/*	 44 */	  69,	 58,	  81,	   37,		7,	   35,
/*	 45 */	  72,	 61,	  85,	   38,		7,	   35,
/*	 46 */	  74,	 64,	  88,	   39,		7,	   50,
/*	 47 */	  77,	 66,	  92,	   40,		7,	   50,
/*	 48 */	  80,	 69,	  96,	   41,		7,	   50

};

static UInt16 tableRows = 0;				// Rows in table
static int topRung = 1;					// Top rung in scrolling area

#define RowRung (topRung + row)			// Item (1-nTable) of top visible row in table

//	setScrollbar  --  Set scrollbar parameters for active view

static void setScrollbar(void)
{
	FormPtr frm = FrmGetActiveForm();
	ScrollBarPtr scroll = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ExerciseRungScrollBar));
	
	SclSetScrollBar(scroll, topRung - 1, 0, nTable - tableRows, tableRows - 1);
}

//	scrollTable  --  Repaint table to reflect change to scrollbar

static void scrollTable(void)
{
	FormPtr f = FrmGetActiveForm();
	TablePtr tp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, ExerciseExerciseLadderTable));

	TblUnhighlightSelection(tp);
	TblEraseTable(tp);
	TblMarkTableInvalid(tp);
	TblDrawTable(tp);
}

/*	exerciseHandleEvent  --  Handle event while the exercise form
							 is displayed.  */
							  

static Boolean exerciseHandleEvent(EventPtr event)
{
	Boolean handled;
#ifdef __GNUC__
	CALLBACK_PROLOGUE
#endif
	handled = false;
	
	switch (event->eType) {
	  	
  		case frmOpenEvent:
  			{
	  			FormPtr f = FrmGetActiveForm();

				FrmDrawForm(f);
			}
			handled = true;
			break;
			
		case frmCloseEvent:
			FrmEraseForm(FrmGetActiveForm());
			FrmDeleteForm(FrmGetActiveForm());
			handled = true;
			break;

		case ctlSelectEvent:
			switch(event->data.ctlSelect.controlID) {
				case ExerciseRecordButton:			// Record button: Enter current rung in today's log entry
				case ExerciseTodayButton:			// Today button: Show entry for today
					{
						DateTimeType dt;
	
						TimSecondsToDateTime(TimGetSeconds(), &dt);
						goToToday = true;
						hdEnsureMonthPresent(dt.year, dt.month - 1);
						logGotoMonth = hdFindMonth(dt.year, dt.month - 1);
						if (event->data.ctlSelect.controlID == ExerciseRecordButton) {
							recordRung = prefs.currentRung;
						}
						FrmGotoForm(MonthlyLogForm);
						handled = true;
					}
					break;
					
				case ExerciseChartButton:			// Chart button: Show chart for month
					{
						DateTimeType dt;
	
						TimSecondsToDateTime(TimGetSeconds(), &dt);
						hdEnsureMonthPresent(dt.year, dt.month - 1);
						logGotoMonth = hdFindMonth(dt.year, dt.month - 1);
						FrmGotoForm(MonthlyChartForm);
					}
					handled = true;
					break;
					
				case ExerciseTrendButton:			// Trend button: Show trend analysis
					FrmGotoForm(TrendAnalysisForm);
					handled = true;
					break;
			}
			break;
			
		case tblEnterEvent:						// Handle tap on row to change current rung
			{
				UInt16 row = event->data.tblEnter.row,
					 col = event->data.tblEnter.column;
				TablePtr tp = event->data.tblEnter.pTable;

				if (!isLabel(RowRung)) {			
					if (prefs.currentRung != IndexToRung(RowRung - 1)) {
						if (prefs.currentRung != 0) {
							int orung = RungToIndex(prefs.currentRung) - (topRung - 1);
							
							//	Only repaint old rung if it's visible in the table
							if ((orung >= 0) && (orung < tableRows)) {
								TblMarkRowInvalid(tp, orung);
							}
						}
						prefs.currentRung = IndexToRung(RowRung - 1);
						TblMarkRowInvalid(tp, row);
						TblRedrawTable(tp);
					}
				}
				handled = true;
			}
			break;

#ifdef SCROLLTRACK
		/*	This code handles on the fly scrolling of the table as the
			scroll bar is moved.  This is disabled by default because
			as of the Palm IIIx era the CPU isn't fast enough to repaint
			the whole table smoothly as you scroll.  */			
		case sclRepeatEvent:
			{
				topRung = event->data.sclRepeat.newValue + 1;
				scrollTable();
				handled = true;
			}
			break;
#else			
		case sclExitEvent:
			{
				topRung = event->data.sclExit.newValue + 1;
				scrollTable();
				handled = true;
			}
			break;
#endif			
		
		case keyDownEvent:
		
			switch (event->data.keyDown.chr) {

				case pageUpChr:
					{	int n = topRung;
					
						n -= tableRows;
						if (n < 1) {
							n = 1;
						}
						topRung = (UInt16) n;
						setScrollbar();
						scrollTable();
						handled = true;
					}
					break;
					
				case pageDownChr:
					topRung += tableRows;
					if (topRung > ((nTable - tableRows) + 1)) {
						topRung = (nTable - tableRows) + 1;
					}
					setScrollbar();
					scrollTable();
					handled = true;
					break;
			}
			break;

#ifdef NEEDED						
		case menuEvent:
			{
				FieldPtr f = getFocusObject();
				
				switch (event->data.menu.itemID) {
					case LogEditUndo:
						if (f != NULL) {
							FldUndo(f);
						}
						handled = true;
						break;
				}
			}
#endif
	}

#ifdef __GNUC__
	CALLBACK_EPILOGUE
#endif
	return handled;
}

//	drawItem  --  Draw exercise table items

static void drawItem(MemPtr table, Int16 row, Int16 col, RectanglePtr rect)
{
	char txt[12];
	int n = IndexToRung(RowRung - 1);
	Boolean crung = n == prefs.currentRung;
	FontID fid;
	TablePtr tp = (TablePtr) table;
	
#ifdef __GNUC__
	CALLBACK_PROLOGUE
#endif

	if (col != C_Rung) {
		n = Exercise(RowRung, col);
	}
	if (isLabel(RowRung)) {
	
		/*  This is a section label row.  Draw the label.  */
		
		if (col == C_Count) {
			char s[60];
			
			stringResource(Exercise(RowRung, C_Bend) == IntroductoryLabel ?
				IntroductoryLadderString : LifetimeLadderString,
				s, sizeof s);

			rect->topLeft.x = tp->bounds.topLeft.x;
			rect->extent.x = tp->bounds.extent.x;
			fid = FntSetFont(boldFont);
			drawInFieldCentre(s, rect);
			FntSetFont(fid);
		}
	} else {
		StrPrintF(txt, "%d", n);
		if (crung) {
			fid = FntSetFont(boldFont);
		}
		drawInFieldRight(txt, rect);
		if (crung) {
			FntSetFont(fid);
		}
	}

#ifdef __GNUC__
	CALLBACK_EPILOGUE
#endif
}

/*	exerciseLoad  --  Initialise exercise form and
					  display.  Called when ApplicationHandleEvent
					  receives a frmLoadEvent for the exercise
					  form. */

void exerciseLoad(FormPtr f)
{
	TablePtr tp = FrmGetObjectPtr(f, FrmGetObjectIndex(f,ExerciseExerciseLadderTable));
	UInt16 i, j, nr;
	
	FrmSetEventHandler(f, exerciseHandleEvent);
	tableRows = nr = TblGetNumberOfRows(tp);
	
	//	Pre-scroll the table so the current selected rung is visible
	
	if (prefs.currentRung > 0) {
		topRung = (RungToIndex(prefs.currentRung) + 1) - (tableRows / 2);
		if (topRung <= 0) {
			topRung = 1;
		} else if ((topRung + tableRows) > nTable) {
			topRung = nTable - (tableRows - 1);
		}
	} else {
		topRung = 1;
	}
	
	//	Set custom draw routine for every column in the table

	TblSetCustomDrawProcedure(tp, C_Rung, drawItem);
	TblSetCustomDrawProcedure(tp, C_Bend, drawItem);
	TblSetCustomDrawProcedure(tp, C_Sit_up, drawItem);
	TblSetCustomDrawProcedure(tp, C_Leg_lift, drawItem);
	TblSetCustomDrawProcedure(tp, C_Push_up, drawItem);
	TblSetCustomDrawProcedure(tp, C_Steps, drawItem);
	TblSetCustomDrawProcedure(tp, C_Count, drawItem);
	
	for (i = 0; i < nr; i++) {
		for (j = 0; j < nColumns; j++) {
			TblSetItemStyle(tp, i, j, customTableItem);
		}
	}

	for (i = 0; i < nColumns; i++) {
		TblSetColumnUsable(tp, i, true);
	}
	
	setScrollbar();
}